import unittest


class A(unittest.TestCase):
    def test_it(self):
        pass


class B(unittest.TestCase):
    def test_it(self):
        pass


class C(unittest.TestCase):
    def test_it(self):
        pass


class D(unittest.TestCase):
    def test_it(self):
        pass
